%% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %%
% <><><><><><><><><><><>   Preprocessing-Batch    <><><><><><><><><><><> %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%                                                 Version:    13.09.2009 %
%                                                     (c) Roland Pfister %
%                                             roland_pfister@t-online.de %
% 1. Synopsis                                                            %
%                                                                        %
% This SPM8 batch contains automated routines for preprocessing a set of %
% fMRI images. All necessary adjustments are marked by a '<-X' symbol to %
% the right of the code. Optionally, you may also run this batch file in %
% prompt mode, i.e. specify all settings on runtime with a dialogue in   %
% the Matlab window. To enable prompt mode, simply change the following  %
% value to 'true':                                                       %
%                                                                        %
    prompt_mode = false;
%                                                                        %
% In either case, the batch need a certain folder setup. All names can   %
% be chosen at will except for the "results" folder, if the resoective   %
% batch is to be used later on.                                          %
%                                                                        %
%   [...]\participant_xy                                                 %
%            \jobs              < Containing this batch file.            %
%            \session folders   < Containing the EPIs; folder            %
%                                 names can be entered below.            %
%            \t1                < T1-image.                              %
%            \results           < For 1st level analysis (optional).     %
%                                                                        %
%                                                                        %
% 2. Content (Sections)                                                  %
%                                                                        %
%       - Settings: Design- and batch specifications.                    %
%       - Start Bacth Processing                                         %
%       - Job 01: Set Working Directory                                  %
%       - Job 02: Coregirstration                                        %
%       - Job 03: Relign & Unwarp                                        %
%       - Job 04: Slicetime Correction                                   %
%       - Job 05: Normalisation                                          %
%       - Job 06: Smoothing                                              %
%       - End Batch and Workspace Cleanup                                %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Settings     <><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %


% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Basic Design Specifications.                                           %
%                                                                        %
TR = 2;                     % TR in seconds.                                <-X (1/12)
n_slices = 33;              % Number of slices.                             <-X (2/12)
TA = TR-(TR/n_slices);      % TA in seconds; computed automatically.

slice_order = 1:33;         % Order for slice timing.                       <-X (3/12)
reference_slice = 17;       % Reference slice for slice timing.             <-X (4/12)
voxel_size = [3 3 3];       % Voxel size for normalisation.                 <-X (5/12)

smoothing_width = [9 9 9];  % FWHM setting for smoothing.                   <-X (6/12)

n_sessions = 3;             % Number of sessions.                           <-X (7/12)
starting_image = [1, 1 ,1]; % Starting and ending image indices             <-X (8/12)
final_image = [381,381,381];% for each session.                             <-X (9/12)
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Image Name Specification.                                              %
%                                                                        %
% The image (EPIs) filenames and paths are concatenated from these       %
% sources:                                                               %
%                                                                        %
%   (1) base_folder         - Path of participant folder. The base fol-  %
%                             folder consists of the experiment folder   %
%                             and a participant-specific subfolder.      %
%   (2) session_names       - Folder specification for session i.        %
%   (3) participant_prefix  - Participant-specific prefix of each .img.  %
%   (4) image number        - A four-digit image number (leading 0s are  %
%                             added automatically)).                     %
%   (5) image suffix        - '.img,1'.                                  %
%                                                                        %
% Example:                                                               %                                                                  
%                                                                        %
%   'C:\SampleStudy\participant_01\session_1\sn_001_0005.img'            %
%    |------------(1)------------| |--(2)--| |-(3)-||-(4,5)-|            %
%                                                                        %
cd ..;                      % The experiment folder is set to the 
exp_folder = [pwd '\'];     % directory one level above this file.
participant_folders = 'pb*';% Identifier for participant folders in
                            % the main directory of the experiment.
                            % Used to construct base_folder for each
                            % participant during batch processing.
session_names = 'session_'; % Name of session folders (excl. numbers).      <-X (10/12)
participant_prefix = ...    % Enter 'folder_name' to use the name of        <-X (11/12)
   'folder_name';           % the participant folder instead.
part_image_sep = '_';       % This symbol will be placed between par-
                            % ticipant number and image index in the
                            % image filenames.
                            
% T1 location.
t1_folder = 't1\';
t1_name = 's*.img';         % The T1 image is read automatically.           <-X -- "I'm the last one!"
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Image Prefixes                                                         %
%                                                                        %
prefix_coreg     = 'r';     %T1
prefix_realign   = 'u';     %EPIs
prefix_st        = 'a';     %EPIs
prefix_normalise = 'w';     %EPIs
prefix_smooth    = 's';     %EPIs
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Set the Stage                                                          %
%                                                                        %
clear matlabbatch;          % Clear batch variable in workspace.
j_c = 0;                    % Initialise job counter.

% Prepare batch for processing multiple participants. To avoid the com- 
% command line prompt, use the following lines instead (you can comment 
% the rest of the section).
% start_participant = 1;
% final_participant = 1;

start_participant = input(['\n\n\n' ...
    '<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
        '<><><><><>\n'...
    '<><><><><>      Preprocessing Batch - (c) Roland Pfister      ' ...
        '<><><><><>\n'...
    '<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
        '<><><><><>\n'...
    '<>                                                            ' ...                                                        
        '        <>\n'...
    '<> Specify the participants to be processed. Give first and '   ...
        'last fol- <>\n'...
    '<> der number starting with participant prefix. See the file'   ...
        ' for more <>\n' ...
    '<> information                                                ' ...                                                        
        '        <>\n'...
    '<>                                                            ' ...                                                        
        '        <>\n\n'...
    'First participant        :        ']);
if ~isnumeric(start_participant)
    disp(' ');
    disp('Please enter a single integer!');
    disp(' ');    
end;
final_participant = input('Last participant         :        ');
if ~isnumeric(final_participant) 
    disp(' ');
    disp('Please enter a single integer!');
    disp(' ');   
end;
start_participant = start_participant + 1;
final_participant = final_participant + 1;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



%% Start Batch Processing      ><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% Get participant directories.
participants = dir([exp_folder participant_folders]);
participant_prefix_setting = participant_prefix;

for participant_index = start_participant:final_participant

base_folder = [exp_folder participants(participant_index).name  '\'];



%Get participant folder name and correct prefix:
if strcmp(participant_prefix_setting,'folder_name') == 1
    participant_prefix = [participants(participant_index). ...
        name part_image_sep];
else
    participant_prefix = [participant_prefix_setting part_image_sep];
end;


% Get T1 and template images.
temp = dir([base_folder t1_folder t1_name]);
t1_path = [base_folder t1_folder temp.name ',1']; % Path to T1 image.
clear temp;
spm_path = strrep(which('spm'),'spm.m','');   % SPM main folder.
template_path = [spm_path 'templates\EPI.nii,1'];
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 01: Set Working Directory    <><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% SPM's working directory is set to the path of the participant folder.  %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.cfg_basicio.cfg_cd.dir = {base_folder};
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 02: Coregistration.    <><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
if starting_image(1) < 10
    temp = '000';
elseif starting_image(1) < 100
    temp = '00';
else
    temp = '0';
end;
matlabbatch{j_c}.spm.spatial.coreg.estwrite.ref = ...
    {[ base_folder session_names '1\' participant_prefix temp ...
       num2str(starting_image(1)) '.img,1']};
clear temp;
matlabbatch{j_c}.spm.spatial.coreg.estwrite.source = {t1_path};
matlabbatch{j_c}.spm.spatial.coreg.estwrite.other = {''};
matlabbatch{j_c}.spm.spatial.coreg.estwrite.eoptions.cost_fun = 'nmi';
matlabbatch{j_c}.spm.spatial.coreg.estwrite.eoptions.sep = [4 2];
matlabbatch{j_c}.spm.spatial.coreg.estwrite.eoptions.tol = ...
    [0.02 0.02 0.02 0.001 0.001 0.001 0.01 0.01 0.01 0.001 0.001 0.001];
matlabbatch{j_c}.spm.spatial.coreg.estwrite.eoptions.fwhm = [7 7];
matlabbatch{j_c}.spm.spatial.coreg.estwrite.roptions.interp = 1;
matlabbatch{j_c}.spm.spatial.coreg.estwrite.roptions.wrap = [0 0 0];
matlabbatch{j_c}.spm.spatial.coreg.estwrite.roptions.mask = 0;
matlabbatch{j_c}.spm.spatial.coreg.estwrite.roptions.prefix = ...
    prefix_coreg;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 03: Realign & Unwarp.    <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
% The data cell array is filled with images.
for i = 1:n_sessions
matlabbatch{j_c}.spm.spatial.realignunwarp.data(i).scans{ ... 
    final_image(i)-starting_image(i)+1,1} = 'preallocation';
end;
for i = 1:n_sessions
    n = 1;
    matlabbatch{j_c}.spm.spatial.realignunwarp.data(i).pmscan = '';
for j = starting_image(i):final_image(i)
    if j < 10
        temp = '000';
    elseif j < 100
        temp = '00';
    else
        temp = '0';
    end;
    matlabbatch{j_c}.spm.spatial.realignunwarp.data(i).scans{n,1} = ...
    [base_folder session_names num2str(i) '\' participant_prefix ...
    temp num2str(j) '.img,1']; %#ok<*SAGROW>
    n = n + 1;
end;
end;

matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.quality = 0.9;
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.sep = 4;
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.fwhm = 5;
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.rtm = 0;
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.einterp = 2;
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.ewrap = [0 0 0];
matlabbatch{j_c}.spm.spatial.realignunwarp.eoptions.weight = {''};
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.basfcn = [12 12];
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.regorder = 1;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.lambda = 100000;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.jm = 0;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.fot = [4 5];
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.sot = [];
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.uwfwhm = 4;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.rem = 1;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.noi = 5;
matlabbatch{j_c}.spm.spatial.realignunwarp.uweoptions.expround = ...
    'Average';
matlabbatch{j_c}.spm.spatial.realignunwarp.uwroptions.uwwhich = [2 1];
matlabbatch{j_c}.spm.spatial.realignunwarp.uwroptions.rinterp = 4;
matlabbatch{j_c}.spm.spatial.realignunwarp.uwroptions.wrap = [0 0 0];
matlabbatch{j_c}.spm.spatial.realignunwarp.uwroptions.mask = 1;
matlabbatch{j_c}.spm.spatial.realignunwarp.uwroptions.prefix = ...
    prefix_realign;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 04: Slicetime Correction.        <><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
for i = 1:n_sessions
matlabbatch{j_c}.spm.temporal.st.scans{1,i}{ ... 
    final_image(i)-starting_image(i)+1,1} = 'preallocation';
end;
for i = 1:n_sessions
    n = 1;
for j = starting_image(i):final_image(i)
    if j < 10
        temp = '000';
    elseif j < 100
        temp = '00';
    else
        temp = '0';
    end;
    if i == 1 && j == 1
        matlabbatch{j_c}.spm.temporal.st.scans{1,1}{1,1} = ...
            [base_folder session_names num2str(1) '\mean'  ...
            prefix_realign participant_prefix temp         ...
            num2str(starting_image(1)) '.img,1'];
        n = n + 1;
    end;
    matlabbatch{j_c}.spm.temporal.st.scans{1,i}{n,1} = ...
        [base_folder session_names num2str(i) '\' prefix_realign ...
        participant_prefix temp num2str(j) '.img,1'];
    n = n + 1;
end;
end;

matlabbatch{j_c}.spm.temporal.st.nslices = n_slices;
matlabbatch{j_c}.spm.temporal.st.tr = TR;
matlabbatch{j_c}.spm.temporal.st.ta = TA;
matlabbatch{j_c}.spm.temporal.st.so = slice_order;
matlabbatch{j_c}.spm.temporal.st.refslice = reference_slice;
matlabbatch{j_c}.spm.temporal.st.prefix = prefix_st;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 05: Normalisation.       <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
temp = '000';
matlabbatch{j_c}.spm.spatial.normalise.estwrite.subj.source = ...
    {[base_folder session_names num2str(1) '\' prefix_st ...
     'mean' prefix_realign participant_prefix temp ...
     num2str(starting_image(1)) '.img,1']};
matlabbatch{j_c}.spm.spatial.normalise.estwrite.subj.wtsrc = '';

matlabbatch{j_c}.spm.spatial.normalise.estwrite.subj.resample{ ... 
    sum(final_image-starting_image)+n_sessions,1} = 'preallocation';
n = 1;
for i = 1:n_sessions
for j = starting_image(i):final_image(i)
    if j < 10
        temp = '000';
    elseif j < 100
        temp = '00';
    else
        temp = '0';
    end;
    if i == 1 && j == 1
        matlabbatch{j_c}.spm.spatial.normalise.estwrite.subj. ...
            resample{n,1} = [base_folder session_names num2str(i) ...
            '\' prefix_st 'mean' prefix_realign  participant_prefix  ...
            temp num2str(starting_image(1)) '.img,1'];
        n = n + 1;
    end;
    matlabbatch{j_c}.spm.spatial.normalise.estwrite.subj.resample ...
        {n,1} = [base_folder session_names num2str(i) '\' prefix_st ...
        prefix_realign participant_prefix temp num2str(j) '.img,1'];
    n = n + 1;
end;
end;

matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.template = ...
    {template_path};
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.weight = '';
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.smosrc = 8;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.smoref = 0;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.regtype = 'mni';
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.cutoff = 25;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.nits = 16;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.eoptions.reg = 1;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.preserve = 0;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.bb = ...
    [-78 -112 -50
      78   76  85];
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.vox = voxel_size;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.interp = 1;
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.wrap = [0 0 0];
matlabbatch{j_c}.spm.spatial.normalise.estwrite.roptions.prefix = ...
    prefix_normalise;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 06: Smoothing.       <><><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.spatial.smooth.data{ ... 
    sum(final_image-starting_image)+n_sessions,1} = 'preallocation';
n = 1;
for i = 1:n_sessions
for j = starting_image(i):final_image(i)
    if j < 10
        temp = '000';
    elseif j < 100
        temp = '00';
    else
        temp = '0';
    end;
    if i == 1 && j == 1
        matlabbatch{j_c}.spm.spatial.smooth.data{n,1}= [base_folder ...
            session_names num2str(i)  '\' prefix_normalise prefix_st ...
            'mean' prefix_realign  participant_prefix  temp ...
            num2str(starting_image(1)) '.img,1'];
        n = n + 1;
    end;
    matlabbatch{j_c}.spm.spatial.smooth.data{n,1} = [base_folder ...
        session_names num2str(i) '\' prefix_normalise prefix_st ...
        prefix_realign participant_prefix temp num2str(j) '.img,1'];
    n = n + 1;
end;
end;

matlabbatch{j_c}.spm.spatial.smooth.fwhm = smoothing_width;
matlabbatch{j_c}.spm.spatial.smooth.dtype = 0;
matlabbatch{j_c}.spm.spatial.smooth.prefix = prefix_smooth;





%% End Batch and Workspace Cleanup.         <><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %

% End Batch Processing.
end;

% Workspace Cleanup.
clear TA;
clear TR;
clear base_folder;
clear t1_folder;
clear t1_name;
clear final_image;
clear i;
clear j;
clear n;
clear n_sessions;
clear n_slices;
clear part_image_sep;
clear participant_prefix;
clear prefix_coreg;
clear prefix_normalise;
clear prefix_realign;
clear prefix_smooth;
clear prefix_st;
clear prompt_mode;
clear reference_slice;
clear session_names;
clear slice_order;
clear smoothing_width;
clear spm_path;
clear starting_image;
clear subject_number;
clear t1_path;
clear temp;
clear template_path;
clear voxel_size;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %