%% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %%
% <><><><><><><><>       Automated PPI Extraction       <><><><><><><><> %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%                                                 Version:    31.01.2009 %
%                                                     (c) Roland Pfister %
%                                             roland_pfister@t-online.de %
% Synopsis                                                               %
%                                                                        %
% This script is a batch clone of spm_peb_ppi.m which is adapted to com- %
% pute (only) psychophsiologic interactions with an automated algorithm. %
%                                                                        %
% Starting with a VOI definition .mat file, PPI structures are extracted %
% as if the PPI button in the interactive window was used. Additionally, %
% all regressors for PPI modeling and analysis (ppi, Y, P) are cumulated %
% over sessions.                                                         %
%                                                                        %
% See the section "Get Input Variables" to adjust the script to your de- %
% mands.                                                                 %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %


%% Excerpt from spm_peb_ppi.m       <><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% SETTING UP A PPI THAT ACCOUNTS FOR THE HRF                             %
% ==================================================================     %
% PPI's were initially conceived as a means of identifying regions whose %
% reponses can be explained in terms of an interaction between activity  %
% in a specified source (the physiological factor) and some experimental %
% effect (psychological factor). However, a problem in setting up PPI's  %
% is that - in order to derive a proper estimate of the interaction be-  %
% tween a psychological variable (P) and measured hemodynamic signal (x) %
% - one cannot simply convolve the psychological variable with the hrf   %
% (HRF) and multiply by the signal.                                      %
%                                                                        %
% [...]                                                                  %
%                                                                        %
% PPI STATISTICAL MODEL                                                  %
% ==================================================================     %
% Once the PPI.ppi interaction term has been calculated,  a new GLM must %
% be set up to search for the interaction effects across the brain. This %
% is done using a standard first level fMRI model, which must include 3  %
% covariates, PPI.ppi (interaction), PPI.Y (main effect: source region   %
% bold signal) and PPI.P (main effect: "psychological" condition), plus  %
% any nuisance regressors according to the particular design.            %
%                                                                        %
% NB: Designs that include only the interaction term without the main    %
% effects are not proper as inferences on the interaction will include a %
% mixture of both main and interaction effects.                          %
%                                                                        %
% Once the model has been setup and run, a contrast of [1 0 0 ] over the %
% PPI.ppi, PPI.Y and PPI.P columns respectively, will show regions with  %
% a positive relationship to the interaction term, discounting any main  %
% effects. Negative regressions can be examined with [-1 0 0].           %
%                                                                        %
% [...]                                                                  %
%                                                                        %
% PPI.ppi    - (PSY*xn  or xn1*xn2) convolved with the HRF               %
% PPI.Y      - Original BOLD eigenvariate.                               %
% PPI.P      - PSY convolved with HRF for psychophysiologic interactions,%
%              or in the case of physiophysologic interactions contains  %
%              the eigenvariate of the second region.                    %
% PPI.name   - Name of PPI                                               %
% PPI.xY     - Original VOI information                                  %
% PPI.xn     - Deconvolved neural signal(s)                              %
% PPI.U.u    - Psychological variable or input function (PPIs only)      %
% PPI.U.w    - Contrast weights for psychological variable (PPIs only)   %
% PPI.U.name - Names of psychological conditions (PPIs only)             %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Get Input Variables        <><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% Input variables:                                                       %
%                                                                        %
%   - SPM.mat               - The participant's results file.            %
%   - VOI definition file   - The voi definition .mat file.              %
%   - Number of sessions    - Number of sessions used in the experiment. %
%   - Contrast vectors      - Contrast weights for each condition.       %
%   - Contrast index        - Index for which contrasts are to be eval-  %
%                             uated.                                     %
%                                                                        %

% Design specification and folder layout.
participants = [1:3,4]; % Participant and contrast indices; the input       <-X (1/8)
contrasts = [2:3];      % has to be a column vector, e.g. [1:3,5,10].       <-X (2/8)
n_sessions = 3;         % Number of sessions in the design (integer).       <-X (3/8)

base_folder = strrep(pwd,'Batch','');
participant_prefix = 'pb_';             % Participant folders prefix.       <-X (4/8)
results_folder = 'results_resp\';       % Individual results folders.       <-X (5/8)


% VOI names for each contrast -with or without coordinates. Enter           <-X (6/8)  
% the name of VOI file (.mat) without  VOI_ prefix and _sessionnr  
% suffix; both will be appended prior to loading the VOI file in 
% question, e.g.
%
%    target_regions{1} = { 'Insula_L' ...
%                          'Insula_R' };
%    target_regions{2} = { 'Insula_R' };
%
% OR (pasted from VOI batch)
%
%    target_regions{1} = { [-36; 17; -2], 'Insula_L'; ...
%                          [ 36; 17; -2], 'Insula_R'; };
%    target_regions{2} = { [ 36; 17; -2], 'Insula_R'; };
%
% ... to calculate a PPI for left and right insula based on con- 
% trast #1 and a PPI for the right insula based on contrast #2.

target_regions{1} = { 'Insula_L' ...
                      'Insula_R' };
target_regions{2} = { 'Insula_R' };

% Set up contrast vectors ("Do you want to include...?"). Contrast          <-X (7/8)
% vectors have to be entered as cell arrays - with contrasts being
% separate numeric arrays (possible elements: -1, 0, or +1) in the 
% columns of contrasts_ppi. The second index specifies the session
% number for the respective contrast vector.
%
% Example:  contrasts_ppi{1,1} = [1 -1 0]; < Contrast 1, Session 1
%           contrasts_ppi{1,2} = [1 0 -1]; < Contrast 1, Session 2
%
% The contrast vector serves two purposes at once: (1) If a '0' is
% entered as contrast weight, the respective condition will not be
% evaluated for the PPI. (2) The non-zero element ('1' or '-1') is
% also used as contrast weight for PPI analysis.
% 
% contrasts_ppi{1,1} = [1 -1 0];
% contrasts_ppi{1,2} = [1 0 -1];
%
% If the same contrast vector is applied to all sessions, use the
% following script:
%
%     for i = 1:size(contrasts_ppi,1)
%     for j = 1:n_sessions
%         contrasts_ppi{i,j} = contrasts_ppi{i,1};
%     end;
%     end;

contrasts_ppi{1,1} = [1 1 1 1 1 1 -1 -1 -1 -1 -1 -1 0 0 0 0 0 0 0];
contrasts_ppi{2,1} = [1 0 1 0 1 0 -1 0 -1 0 -1 0 0 0 0 0 0 0 0];
contrasts_ppi{3,1} = [0 1 0 1 0 1 0 -1 0 -1 0 -1 0 0 0 0 0 0 0];
for i = 1:size(contrasts_ppi,1)
for j = 1:n_sessions
    contrasts_ppi{i,j} = contrasts_ppi{i,1};
end;
end;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Run the algorithm for each specified contrast/session.       <><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %

fprintf('\n\n');
fprintf(['<><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
    '<><><><><><>\n<><><><><><><><>          Starting PPI Analysis  ' ...
    '     <><><><><><><><>\n<><><><><><><><><><><><><><><><><><><><>' ...
    '<><><><><><><><><><><><><><><>\n']);

for participant_index = 1:size(participants,2)
    
participant = participants(participant_index);
if participant < 10
    prefix_correction = '0';
else
    prefix_correction = '';
end;

fprintf('\n\n');
fprintf('<> Participant: %s\n\n',[prefix_correction ...
    num2str(participant) ...
    '   <><><><><><><><><><><><><><><><><><><><><><><><>']);

participant_directory = [base_folder participant_prefix ...
    prefix_correction num2str(participant) '\'];

cd(participant_directory)
cd(results_folder)

try 
    cd VOIs
    voi_directory = pwd;
    cd ..
catch
    error('No VOI files found.');
end;

try
    cd PPIs
catch
    mkdir('PPIs');
    cd PPIs
end;

ppi_directory = pwd;



for contrast_index = 1:size(contrasts,2)

cd(ppi_directory)

contrast = contrasts(contrast_index);
if contrast < 10
    contrast_correction  = '00';
elseif contrast < 100
    contrast_correction  = '0';  
else
    contrast_correction  = '';  
end;

fprintf('\t< Contrast: %s >\n',[contrast_correction num2str(contrast)]);

try
    cd(['con_' contrast_correction num2str(contrast)])
catch
    mkdir(['con_' contrast_correction num2str(contrast)]);
    cd(['con_' contrast_correction num2str(contrast)])
end;

contrast_directory = pwd;
    
for region_index = 1:size(target_regions{contrast},1)

load([participant_directory results_folder 'SPM.mat']);

if size(target_regions{contrast},2) == 2
    voi_name = target_regions{contrast}{region_index,2};
else
    voi_name = target_regions{contrast}{region_index,1}; 
end;

fprintf('\tRegion: %s\n',voi_name);
cd(contrast_directory);
    
for session = 1:n_sessions

%% Get VOI file and create appropriate subfolder.
load([voi_directory '\con_' contrast_correction  num2str(contrast) ...
   '\' 'VOI_' voi_name '_' num2str(session) '.mat']);
if exist(['PPI_' voi_name],'dir') ~= 7
    mkdir(['PPI_' voi_name]);
end;



%% Field 1: PPI.name. The PPI name is concatenated from (1) the VOI
% name, (2) _c[contrast number], and (3) _s[session number].
PPI.name = [voi_name '_c' num2str(contrast) ...
    '_s' num2str(session)];




%% Field 2: PPI.Y. In this script, Y and X0 are taken directly from 
% the VOI file. In the original spm_peb_ppi, Y is extracted from
% the VOI's xY structure (xY.u) without noteable effect (cf. lines
% 204-217 prior to version r3408, or 317-321 in later releases).
%
% The right hand part of the equation removes confounds in Y.
PPI.Y = Y - xY.X0*inv(xY.X0'*xY.X0)*xY.X0'*Y; %#ok<MINV>



%% Variables for deconvolution are set up.
% set up variables from spm.mat
RT     = SPM.xY.RT;
dt     = SPM.xBF.dt;
NT     = RT/dt;
Sess   = SPM.Sess(xY(1).Sess);
   
% get 'causes' or inputs U
u      = length(Sess.U);
U.name = {};
U.u    = [];
U.w    = [];
for  i = 1:u
for  j = 1:length(Sess.U(i).name)
    if contrasts_ppi{contrast,session}(1,i) ~= 0
        U.u             = [U.u Sess.U(i).u(33:end,j)];
        U.name{end + 1} = Sess.U(i).name{j};
        U.w             = [U.w contrasts_ppi{contrast,session}(1,i)];
    end
end
end
PPI.psy = U;

% Setup variables
N     = length(xY(1).u);
k     = 1:NT:N*NT;              % microtime to scan time indices

% create basis functions and hrf in scan time and microtime
hrf   = spm_hrf(dt);

% create convolved explanatory {Hxb} variables in scan time
xb    = spm_dctmtx(N*NT + 128,N);
Hxb   = zeros(N,N);
for i = 1:N
    Hx       = conv(xb(:,i),hrf);
    Hxb(:,i) = Hx(k + 128);
end
xb    = xb(129:end,:);

% get confounds (in scan time) and constant term
X0    = xY(1).X0;
M     = size(X0,2);

% specify covariance components; assume neuronal response is white
% treating confounds as fixed effects
Q      = speye(N,N)*N/trace(Hxb'*Hxb);
Q      = blkdiag(Q, speye(M,M)*1e6  );

% get whitening matrix (NB: confounds have already been whitened)
W      = SPM.xX.W(Sess.row,Sess.row);

% create structure for spm_PEB
clear P;
P{1}.X = [W*Hxb X0];        %#ok<*SAGROW> Design matrix for lowest level
P{1}.C = speye(N,N)/4;      % i.i.d assumptions
P{2}.X = sparse(N + M,1);   % Design matrix for parameters (0's)
P{2}.C = Q;



%% Compute Psychophysiologic Interactions
%
C       = spm_PEB(Y,P);
xn      = xb*C{2}.E(1:N);
xn      = spm_detrend(xn);

% setup psychological variable from inputs and contast weights
PSY     = zeros(N*NT,1);
for i = 1:size(U.u,2)
    PSY = PSY + full(U.u(:,i)*U.w(:,i));
end
%PSY     = spm_detrend(PSY);
% Important change in "Updates_r3408":
%   "PSY = spm_detrend(PSY);  <- removed centering of psych variable
%   prior to multiplication with xn. Based on discussion with Karl
%   and Donald McLaren. "

% multiply psychological variable by neural signal
PSYxn   = PSY.*xn;

% convolve and resample at each scan for bold signal
ppi     = conv(PSYxn,hrf);
ppi     = ppi(k);

% similarly for psychological effect
PSYHRF  = conv(PSY,hrf);
PSYHRF  = PSYHRF(k);

%% Fields 3-6: Save Psychological Variables.
PPI.psy = U;
PPI.P   = PSYHRF;
PPI.xn  = xn;
PPI.ppi = spm_detrend(ppi);
    
    

%% Fields 7-8: Setup Other Output Variables.
PPI.xY  = xY;
PPI.dt  = dt;



%% Save PPI Structure and Clear Workspace       <><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
save([contrast_directory '\' 'PPI_' voi_name '\PPI_' voi_name '_c' ...
    num2str(contrast) '_s' num2str(session) '.mat'],'PPI');

clear C;
clear Hx;
clear Hxb;
clear M;
clear N;
clear NT;
clear P;
clear PPI;
clear PSY;
clear PSYHRF;
clear PSYxn;
clear Q;
clear RT;
clear Sess;
clear U;
clear W;
clear X0;
clear Y;
clear dt;
clear hrf;
clear i;
clear j;
clear k;
clear ppi;
clear u;
clear xY;
clear xb;
clear xn;
end;





%% Compute and Save Cumulative Regressors       <><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% The regressors ppi, Y, and P are cumulated across sessions for each    %
% contrast.                                                              %
%                                                                        %
%for contrast = start_contrast:final_contrast
cd([contrast_directory '\' 'PPI_' voi_name]);
    Y = [];
    P = [];
    ppi = [];
    for session = 1:n_sessions
        load([contrast_directory '\PPI_' voi_name '\PPI_' voi_name '_c' ...
        num2str(contrast) '_s' num2str(session) '.mat']);
        Y = [Y; PPI.Y]; %#ok<*AGROW>
        P = [P; PPI.P];
        ppi = [ppi; PPI.ppi];
        clear PPI;
    end;
    save([contrast_directory '\' 'PPI_' voi_name '\PPI_' voi_name '_c' ...
    num2str(contrast) '.mat'],'Y','P','ppi');

%     % Additionally, a new output folder is created for PPI model 
%     % estimation.
%     if exist(['VOI_' voi_name 'PPI_' voi_name '_c' ...
%             num2str(contrast)],'dir') ~= 7
%         mkdir(['VOI_' voi_name '\PPI_' voi_name '_c' ...
%             num2str(contrast)]);
%     end;
                 
%% Factorial Design Specification.
% The data cell array is filled with images.  
matlabbatch{1}.spm.stats.factorial_design.des.mreg.scans = ...
    cell(size(SPM.xY.P,1),1);
for i = 1:size(SPM.xY.P,1)
    matlabbatch{1}.spm.stats.factorial_design.des.mreg.scans{i,1} = ...
         SPM.xY.P(i,:);
end;

temp_design_dir = [contrast_directory '\PPI_' voi_name];
matlabbatch{1}.spm.stats.factorial_design.dir = {temp_design_dir};
matlabbatch{1}.spm.stats.factorial_design.des.mreg.mcov = ...
    struct('c', {}, 'cname', {}, 'iCC', {});
matlabbatch{1}.spm.stats.factorial_design.des.mreg.incint = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(1).c = ppi;
matlabbatch{1}.spm.stats.factorial_design.cov(1).cname = 'PPI';
matlabbatch{1}.spm.stats.factorial_design.cov(1).iCFI = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(1).iCC = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(2).c = Y;
matlabbatch{1}.spm.stats.factorial_design.cov(2).cname = 'Y';
matlabbatch{1}.spm.stats.factorial_design.cov(2).iCFI = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(2).iCC = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(3).c = P;
matlabbatch{1}.spm.stats.factorial_design.cov(3).cname = 'P';
matlabbatch{1}.spm.stats.factorial_design.cov(3).iCFI = 1;
matlabbatch{1}.spm.stats.factorial_design.cov(3).iCC = 1;
matlabbatch{1}.spm.stats.factorial_design.masking.tm.tm_none = 1;
matlabbatch{1}.spm.stats.factorial_design.masking.im = 1;
matlabbatch{1}.spm.stats.factorial_design.masking.em = ...
    {[participant_directory results_folder 'mask.img,1']};
matlabbatch{1}.spm.stats.factorial_design.globalc.g_omit = 1;
matlabbatch{1}.spm.stats.factorial_design.globalm.gmsca.gmsca_no = 1;
matlabbatch{1}.spm.stats.factorial_design.globalm.glonorm = 1;
    
spm_jobman('run',matlabbatch)
clear matlabbatch;

% Model estimation
matlabbatch{1}.spm.stats.fmri_est.spmmat = {[temp_design_dir '\SPM.mat']};
matlabbatch{1}.spm.stats.fmri_est.method.Classical = 1;

spm_jobman('run',matlabbatch)
clear matlabbatch;


load([temp_design_dir '\SPM.mat']);
copyfile([temp_design_dir '\SPM.mat'],[temp_design_dir '\SPM.backup.mat']);
% Dummy-contrast erzeugen, damit fc_util funktioniert (nach dem Mailing-
% list-Comment von Iris Trinkler vom 03.02.2005 (Item# 018719)).
try
    iX0  = [SPM.xX.iB SPM.xX.iG];
catch %#ok<*CTCH>
    iX0  = [];
end
SPM.xCon = spm_FcUtil('Set','effects of interest','F','iX0',...
    iX0,SPM.xX.xKXs);

% Wichtig: contrasts(1,i).c muss scheinbar ein row vector sein und kein
% column vector wie in der Mailinglist beschrieben.
SPM.xCon(1,1).name = ['PPI_' voi_name];
SPM.xCon(1,1).STAT = 'T';
SPM.xCon(1,1).c    = [1 0 0 0];
SPM.xCon(1,1).iX0  = 'c';
SPM.xCon(1,1) = spm_FcUtil('Set',['PPI_' voi_name],...
   'T','c',[1 0 0 0].',SPM.xX.xKXs); 
spm_contrasts(SPM,1);
save([temp_design_dir '\SPM.mat'],'SPM');
clear temp_design_dir;
clear iX0;
clear P;

end; % Regions loop.

clear region_index;
clear voi_name;
clear session;
    
end; % Contrasts loop.

clear contrast;
clear contrast_correction;
clear contrast_directory;
clear participant_directory;
clear voi_directory;

end; % Participant loop.




%% Clear Workspace              <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
clear results_folder;
clear base_folder;
clear n_sessions;
clear target_regions;
clear ppi_directory;
clear ppi_dir;
clear participant;
clear participant_index;
clear participant_prefix;
clear prefix_correction;
clear participants;
clear contrasts;
clear contrasts_ppi;
clear contrast_index;
clear participant_string;


clear i;
clear j;
clear Y;
clear P;
clear ppi;
clear PPI;
clear SPM;
clear ans;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %

%clear;